@@ -17,10 +17,23 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+/* Changelog:
+ * August 25, 2002:
+ * Daniel Sterling (dan@lost-habit.com):
+   added #include <unistd.h>
+   removed unused variables
+   down variable set when hd is down instead of vice versa
+   don't go into daemon mode if timeout is 30 seconds or less
+   warn if timeout is less than 60 seconds.... that's still very low--
+    I recommend timeouts of at least 20 minutes
+   fixed daemon to spin down drive more than just one time (oops!!)
+ */
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
 #include <fcntl.h>
+#include <unistd.h>
 #include <sys/ioctl.h>
 #include <sys/stat.h>
 #include <linux/major.h>
@@ -50,9 +63,8 @@ void handler()
 
 int main(int argc, char *argv[])
 {
-    int result;
     int fd;
-    int down = 1;
+    int down = 0;
     struct stat statbuf;
 
     if (argc < 2 || argc > 3) {
@@ -91,8 +103,8 @@ int main(int argc, char *argv[])
 	if (last == 0)
 	    printf("%s has not been accessed yet\n", argv[1]);
 	else
-	    printf("%s has been idle for %d second%s\n", argv[1],
-		   last, (last == 1 ? "" : "s"));
+	    printf("%s has been idle for %i second%s\n", argv[1],
+		   (int) last, (last == 1 ? "" : "s"));
     } else {
 	long timeout;
 
@@ -101,9 +113,11 @@ int main(int argc, char *argv[])
 	    close(fd);
 	    exit(1);
 	}
-	if (timeout == 0) {
-	    long off = 0;
-
+	if (timeout <= 30) {
+	    if (timeout != 0) fprintf(stderr,
+	        "Really low timeouts are a bad, "
+	        "bad idea.\nI'll spin down immediately for you. Use a higher "
+	        "timeout for daemon mode.");
 	    /* Spin down disk immediately */
 
 	    if (ioctl(fd, SCSI_IOCTL_STOP_UNIT) < 0) {
@@ -115,19 +129,22 @@ int main(int argc, char *argv[])
 	    FILE *fp;
 	    int dev;
 	    long last;
+	    long llast = 0;
 	    pid_t pid;
 
 	    /* Spin down disk after an idle period */
-	    if (timeout < 30)
-		fprintf(stderr, "Warning: timeouts less than 30 seconds are really not such a good idea\n");
+	    if (timeout < 60)
+		fprintf(stderr,
+			"Warning: timeouts less than 60 seconds are really "
+			"not such a good idea..\n");
 
 	    /* This is ugly, but I guess there is no portable way to do this */
 	    dev = ((statbuf.st_rdev >> 4) & 15) + 'a';
 
-	    sprintf(lockfn, "/var/run/scsi-idle.sd%c.pid", dev);
+	    sprintf(lockfn, getenv("PIDFILE") ? getenv("PIDFILE") : "/var/run/scsi-idle.sd%c.pid", dev);
 
 	    if ((fp = fopen(lockfn, "r")) != NULL) {
-		fscanf(fp, "%ld", &pid);
+		fscanf(fp, "%i", &pid);
 		fclose(fp);
 
 		kill(pid, SIGTERM);
@@ -149,11 +166,21 @@ int main(int argc, char *argv[])
 		    close(fd);
 		    exit(1);
 		}
-		fprintf(fp, "%ld\n", getpid());
+		fprintf(fp, "%i\n", getpid());
 		fclose(fp);
 
 		nice(10);
 
+/* main daemon loop.
+ * obtain idle time.
+ * have we shut down the drive already?
+   * IF yes: has the idle time reset since the last check?
+     * IF yes: reset our drive status variable
+ * IF the drive is running AND idle time > user-specified timeout, 
+   send STOP_UNIT and set our status variable.
+ * IF idle time < timeout, sleep until idle time would be > timeout.
+ * ELSE sleep for timeout.
+ */
 		for (;;) {
 		    if ((last = ioctl(fd, SD_IOCTL_IDLE, &last)) < 0) {
 			perror(argv[1]);
@@ -167,25 +194,28 @@ int main(int argc, char *argv[])
 			printf("%s has been idle for %d second%s\n", argv[1],
 			       last, (last == 1 ? "" : "s"));
 #endif
-
-		    if (last > timeout) {
-			if (down && (ioctl(fd, SCSI_IOCTL_STOP_UNIT) != 0)) {
+		    if (down == 1 && llast >= last)
+		        down = 0;
+		    llast = last;
+		    if (last >= timeout) {
+			if (down == 0 &&
+			(ioctl(fd, SCSI_IOCTL_STOP_UNIT) != 0)) {
 			    perror(argv[1]);
 			    close(fd);
 			    exit(1);
 			}
 			last = 0;
-			down = 0;
+			down = 1;
 		    }
 #if DEBUG
-		    printf("sleeping for %d seconds\n", timeout - last + 1);
+		    printf("sleeping for %d seconds\n", timeout - last);
 #endif
-		    sleep(timeout - last + 1);
+		    sleep(timeout - last);
 		}
-		/* notreached */
+		/* not reached */
 
 	    default:
-		printf("%s: idle daemon started, timeout is %ld seconds\n",
+		getenv("BEQUIET") || printf("%s: idle daemon started, timeout is %ld seconds\n",
 		       argv[1], timeout);
 		/* Ok, done */
 		break;
